from pathlib import Path
from sys import platform

from pytest import mark

from venvflon import utils


@mark.slow
@mark.skipif(condition=platform != 'win32', reason='Run only on Windows')
@mark.parametrize('cmd, result', [('Clear-Host', 0), ('bullshit', -1)])
def test_run_command(cmd, result):
    rc = utils.run_command(cmd=['powershell', cmd])
    assert rc == result

def test_get_command_output_success():
    rc, err, out = utils.get_command_output(cmd=['python', '-V', '-V'])
    assert rc == 0
    assert err == ''
    assert 'Python 3.' in out


def test_get_command_output_call_process_error():
    rc, err, out = utils.get_command_output(cmd=['python', '-fake'])
    assert rc == 2
    assert 'Unknown option: -f' in err
    assert out == ''


def test_get_command_output_file_not_found_error():
    rc, err, out = utils.get_command_output(cmd=['fake', '-fake'])
    assert rc == 255
    assert err == 'None of venv were detected'
    assert out == ''


@mark.ci
def test_make_and_remove_sym_link():
    new_sym_link = Path(__file__).parent / 'new'
    utils.make_sym_link(to_path=new_sym_link, target=Path(__file__), mode=utils.LinkMode.PYTHON)
    assert new_sym_link.is_symlink()
    assert not new_sym_link.is_dir()
    if platform == 'linux':
        assert new_sym_link.is_file()
    elif platform == 'win32':
        assert not new_sym_link.is_file()
    utils.rm_sym_link(sym_link=new_sym_link, mode=utils.LinkMode.PYTHON)
    assert not new_sym_link.exists()


def test_success_deep_1_venv_list_in(resources):
    venvs = utils.venv_list_in(current_path=resources, max_depth=1)
    assert len(venvs) == 3
    assert sorted([venv.name for venv in venvs]) == ['.venv_310', '.venv_311', '.venv_312']


def test_success_deep_2_venv_list_in(resources):
    venvs = utils.venv_list_in(current_path=resources, max_depth=2)
    assert len(venvs) == 4
    assert sorted([venv.name for venv in venvs]) == ['.venv_310', '.venv_311', '.venv_312', '.venv_39']


def test_failure_venv_list_in(resources):
    venvs = utils.venv_list_in(current_path=resources / '.venv10', max_depth=1)
    assert len(venvs) == 0



from __future__ import annotations

from collections.abc import Sequence
from enum import Enum
from os import sep, walk
from pathlib import Path
from subprocess import CalledProcessError, run
from time import sleep


class LinkMode(Enum):
    """Mode to create symbolic link."""
    PYTHON = 'python.exe'
    PWSH5 = 'powershell.exe'
    PWSH7 = 'pwsh.exe'


LINK_MODE_MAP = {0: LinkMode.PYTHON, 5: LinkMode.PWSH5, 7: LinkMode.PWSH7}


def run_command(cmd: Sequence[str], cwd: Path | None = None) -> int:
    """
    Run command in shell as a subprocess.

    :param cmd: The command to be executed as a sequence of strings
    :param cwd: current working directory
    :return: The return code of command
    """
    try:
        proc = run(cmd, check=True, shell=False, cwd=cwd)
        return proc.returncode
    except CalledProcessError as e:
        print(f'Result: {e}')
        return -1


def get_command_output(cmd: Sequence[str], cwd: Path | None = None) -> tuple[int, str, str]:
    """
    Execute command and return its output.

    :param cmd: The command to be executed as a sequence of strings
    :param cwd: current working directory
    :return: Tuple with return code, stderr and stdout
    """
    try:
        result = run(cmd, capture_output=True, check=True, cwd=cwd)
        return result.returncode, result.stderr.decode('utf-8'), result.stdout.decode('utf-8')
    except CalledProcessError as e:
        print(f'Result: {e}')
        return e.returncode, e.stderr.decode('utf-8'), e.stdout.decode('utf-8')
    except (CalledProcessError, FileNotFoundError) as e:
        print(f'Result: {e}')
        return 255, 'None of venv were detected', ''


def make_sym_link(to_path: Path, target: Path, mode: LinkMode = LinkMode.PWSH5, timer: float = 1.2) -> None:
    """
    Make a symbolic link.

    :param to_path: Path to a symbolic link
    :param target: Target path
    :param mode: Method to create symbolic name
    :param timer: sleeping timer for PowerShell5 and 7 option
    """
    if mode == LinkMode.PYTHON:
        to_path.symlink_to(target=target, target_is_directory=True)
    else:
        cmd_symlink = f'"New-Item -ItemType SymbolicLink -Path \\"{to_path}\\" -Target \\"{target}\\"'
        ps_command = f"Start-Process {mode.value} -ArgumentList '-Command {cmd_symlink}' -Verb RunAs"
        print(f'Make symbolic link: {ps_command}')
        run_command(cmd=[mode.value, '-Command', ps_command])
        sleep(timer)


def rm_sym_link(sym_link: Path, mode: LinkMode = LinkMode.PWSH5) -> None:
    """
    Remove a symbolic link.

    :param sym_link: Path to a symbolic link
    :param mode: How to remove a symbolic link
    """
    if mode == LinkMode.PYTHON:
        sym_link.unlink(missing_ok=True)
    else:
        rm_symlink = f"(Get-Item '{sym_link}').Delete()"
        ps_command = f'Start-Process {mode.value} -ArgumentList "-Command {rm_symlink}" -Verb RunAs'
        print(f'Execute: {ps_command}')
        run_command(cmd=[mode.value, '-Command', ps_command])


def venv_list_in(current_path: Path, max_depth: int = 1) -> list[Path]:
    """
    Find all virtual environments in a given path.

    :param current_path: Path to search in
    :param max_depth: Maximum depth of search
    :return: A list of paths to virtual environments
    """
    result = []
    for dir_path, dir_names, _ in walk(current_path, topdown=True):
        for dirname in dir_names:
            if '.venv_' in dirname:
                result.append(Path(dir_path) / dirname)
        if dir_path.count(sep) - str(current_path).count(sep) == max_depth - 1:
            del dir_names[:]
    return result



from __future__ import annotations

from argparse import ArgumentParser
from os import environ
from pathlib import Path
from sys import base_prefix

from venvflon.flon import Gui, __version__

environ['TCL_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tcl8.6')
environ['TK_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tk8.6')
import tkinter as tk

from tkinterdnd2 import TkinterDnD


def run() -> None:
    """Run the main GUI."""
    parser = ArgumentParser(description='simple virtual environment switcher')
    parser.add_argument('-V', '--version', action='version', version='%(prog)s version: ' + __version__)
    parser.add_argument('-p', '--pwsh', action='store', dest='link_mode', type=int, choices=[0, 5, 7], default=5,
                        help='0 - use Python, pwsh5 or pwsh7 to make symbolic link')
    parser.add_argument('-t', '--timer', action='store', dest='timer', type=float, default=1.2,
                        help='sleep time (in seconds) for pwsh5 and pwsh7')
    args = parser.parse_args()

    root_tk = TkinterDnD.Tk()
    width, height = 300, 150
    root_tk.title(f'venvflon - v{__version__}')
    root_tk.geometry(f'{width}x{height}')
    root_tk.iconphoto(False, tk.PhotoImage(file=Path(__file__).parent / 'img' / 'cannula_64.png'))
    gui = Gui(master=root_tk, cli_args=args)
    gui.mainloop()


if __name__ == '__main__':
    run()



from __future__ import annotations

from argparse import Namespace
from os import chdir, environ, getcwd
from pathlib import Path
from sys import base_prefix

from tkinterdnd2 import DND_FILES, TkinterDnD

from venvflon import utils

environ['TCL_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tcl8.6')
environ['TK_LIBRARY'] = str(Path(base_prefix) / 'tcl' / 'tk8.6')
import tkinter as tk

__version__ = '0.5.0'


class Gui(tk.Frame):
    """Tkinter GUI for venvflon."""

    def __init__(self, master: tk.Tk, cli_args: Namespace) -> None:
        """
        Tkinter  GUI for venvflon.

        :param master: Tkinter root
        :param cli_args: CLI arguments
        """
        super().__init__(master)
        self.master: tk.Tk = master
        self.config: Namespace = cli_args  # type: ignore[assignment]
        self.config.link_mode = utils.LINK_MODE_MAP[cli_args.link_mode]
        self.venv = tk.StringVar(value=' ')
        self.status_txt = tk.StringVar()
        self.cwd_entry = tk.StringVar()
        self.cwd_entry.set(getcwd())
        self.venv_list = utils.venv_list_in(current_path=Path(getcwd()))
        self.frame = tk.Frame(master=self.master, relief=tk.GROOVE, borderwidth=2)
        self.status = tk.Label(master=self.master, textvariable=self.status_txt)
        self.cwd = tk.Entry(master=self.master, textvariable=self.cwd_entry, width=20, relief=tk.SUNKEN, font=('Arial', 9))
        self.frame = tk.Frame(master=self.master, relief=tk.GROOVE, borderwidth=2, bg='white')
        self.status = tk.Label(master=self.master, textvariable=self.status_txt, font=('Arial', 9, 'italic'))
        self.cwd.drop_target_register(DND_FILES)   # type: ignore[attr-defined]
        self.init_widgets()

    def init_widgets(self) -> None:
        """Initialize widgets."""
        cwd_label = tk.Label(self.master, text='cwd:', font=('Arial', 9))
        cwd_label.grid(row=0, column=0, sticky=tk.W, padx=5, pady=5)
        self.cwd.grid(row=0, column=1, sticky=tk.W, padx=5, pady=5)
        self.cwd.bind('<Return>', self.refresh_cwd)
        self.cwd.dnd_bind('<<Drop>>', self.drop_in_cwd)  # type: ignore[attr-defined]
        self.add_venvs()
        self.resize_window()

    def add_venvs(self) -> None:
        """Add venvs as radio buttons to the GUI."""
        self._remove_old_radiobuttons()
        if len(self.venv_list):
            self.frame.grid(row=2, column=0, columnspan=3, padx=5, pady=5, sticky=tk.W)
            for i, text in enumerate(self.venv_list, 1):
                rb_venvs = tk.Radiobutton(master=self.frame, text=str(text), variable=self.venv, value=text,
                                          bg='white', font=('Arial', 9), anchor=tk.W, justify=tk.LEFT)
                self._select_current_venv(venv_path=str(text))
                rb_venvs.configure(command=self.venv_selected)
                rb_venvs.grid(row=i, column=1, pady=0, padx=2, sticky=tk.W)
        self.status.grid(row=3, column=0, columnspan=3, sticky=tk.W, padx=5, pady=10)
        self.update_status()

    def _remove_old_radiobuttons(self) -> None:
        """Remove old Radio buttons for venvs."""
        for venv_rb in self.frame.grid_slaves():
            venv_rb.destroy()

    def _select_current_venv(self, venv_path: str) -> None:
        """
        Select the radio button for venv which symlink point to.

        :param venv_path: Path to the venv as string
        """
        sym_link = Path(getcwd()) / '.venv'
        if sym_link.exists() and sym_link.resolve().name in venv_path:
            self.venv.set(venv_path)

    def refresh_cwd(self, *args: tk.Event[tk.Entry]) -> None:
        """
        Refresh the current working directory.

        :param args: Internal tkinter arguments
        """
        new_cwd = Path(self.cwd_entry.get())
        self.cwd.configure(width=len(str(new_cwd)))
        chdir(new_cwd)
        self.master.title(f'venvflon - {new_cwd.name}')
        self.venv_list = utils.venv_list_in(current_path=new_cwd)
        self.add_venvs()
        self.resize_window()

    def drop_in_cwd(self, event: TkinterDnD.DnDEvent) -> None:
        """
        Insert dropped directory into cwd entry.

        :param event: Drop and Drag event
        """
        self.cwd.delete(0, tk.END)
        self.cwd.insert(tk.END, event.data)
        self.refresh_cwd()

    def venv_selected(self) -> None:
        """Set the selected venv as the active one."""
        new_venv = self.venv.get()
        sym_link = Path(getcwd()) / '.venv'
        utils.rm_sym_link(sym_link=sym_link, mode=self.config.link_mode)
        utils.make_sym_link(to_path=sym_link, target=Path(new_venv), mode=self.config.link_mode, timer=self.config.timer)
        self.update_status()

    def resize_window(self) -> None:
        """Resize the window based on the venv list length."""
        venv_txt_length = 30 if not len(self.venv_list) else len(str(self.venv_list[0]))
        venv_txt_height = 2 if not len(self.venv_list) else len(self.venv_list)
        new_width, new_height = venv_txt_length + 300, venv_txt_height * 55 + 17
        self.cwd.configure(width=len(self.cwd_entry.get()))
        self.master.geometry(f'{new_width}x{new_height}')
        self.master.minsize(width=new_width, height=new_height)

    def update_status(self) -> None:
        """Update the status text."""
        _, err, out = utils.get_command_output(cmd=[r'.venv\Scripts\python.exe', '-V'])
        if out:
            self.status_txt.set(f'v{__version__}   /   Current: {out.strip()}')
        elif err:
            self.status_txt.set(f'v{__version__}   /   Error: {err.strip()}')
